unit DemoMain2;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ColGrid;

Const
  RowCount = 10 ;

type
  MonthArray = Array [1..12, 1..RowCount] Of Currency ;
  QuarterArray = Array [1..4, 1..RowCount] Of Currency ;
  TotalArray = Array [1..RowCount] Of Currency ;

  TForm1 = class(TForm)
    Grid: TColossalGrid;
    Procedure GridUnboundFetch(Sender: TObject; Zone, Column: Integer;
      RecordNumber: Longint ; Var Value : string) ;
    procedure FormCreate(Sender: TObject);
    procedure GridValidateCell(Sender: TObject; Text: string;
      var Cancel: Boolean);
    Procedure GridUnboundEditFetch(Sender: TObject; Zone, Column: Integer;
      RecordNumber: Longint ; var Value: string) ;

  private
    Months : MonthArray ;
    Quarters : QuarterArray ;
    Totals : TotalArray ;

    Procedure Recalculate ;
  public
    { Public declarations }

  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

Procedure TForm1.GridUnboundFetch(Sender: TObject; Zone, Column: Integer;
  RecordNumber: Longint ; Var Value : string) ;
  Procedure FillYear ;
    Begin
    Value := IntToStr (1992 + RecordNumber) ;
    End ;

  Procedure FillValue ;
    Begin
    If Column In [1..4] Then
      Begin
      Value := Format ('%m', [Quarters [Column, RecordNumber]]) ;
      End
    Else
      Begin
      Value := Format ('%m', [Months [Column - 4, RecordNumber]]) ;
      End ;
    End ;

  Procedure FillTotal ;
    Begin
    Value := Format ('%m', [Totals [RecordNumber]]) ;
    End ;

  begin
  Case Zone Of
    1: FillYear ;
    2: Fillvalue ;
    3: FillTotal ;
    End ;
  end;

procedure TForm1.FormCreate(Sender: TObject);
  begin
  Grid.RecordCount := RowCount ;
  ReCalculate ;
  end;

procedure TForm1.GridValidateCell(Sender: TObject; Text: string;
                                  var Cancel: Boolean);
  Var
    Value : Currency ;
  begin
  Try
    Value := StrToCurr (Text) ;
    If Grid.ColumnIndex In [5..26] Then
      Begin
      Months [Grid.ColumnIndex - 4, Grid.RecordIndex] := Value ;
      Recalculate ;
      Grid.Refresh ;
      End ;
  Except
    Cancel := True ;
    End ;

  end;


procedure TForm1.Recalculate ;
  Var
    II : Integer ;
    JJ : Integer ;
    KK : Integer ;
    Total : Currency ;
  Begin
  Total := 0 ;
  For KK := 1 To RowCount Do
    Begin
    Totals [KK] := 0 ;
    For JJ := 1 To 4 Do
      Begin
      Quarters [JJ, KK] := 0 ;
      For II := (3 * JJ) - 2 To 3 * JJ Do
        Begin
        Quarters [JJ, KK] := Quarters [JJ, KK] + Months [II, KK] ;
        End ;
      Totals [KK] := Totals [KK] + Quarters [JJ, KK] ;
      End ;
    Total := Total + Totals [KK] ;
    Grid.Zones [3].Columns [1].FooterText := Format ('%m', [Total]) ;
    End ;

  For JJ := 1 To 4 Do
    Begin
    Total := 0 ;
    For KK := 1 To RowCount Do
      Begin
      Total := Total + Quarters [JJ, KK] ;
      End ;
    Grid.Zones [2].Columns [JJ].FooterText := Format ('%m', [Total]) ;
    End ;

  For JJ := 1 To 12 Do
    Begin
    Total := 0 ;
    For KK := 1 To RowCount Do
      Begin
      Total := Total + Months [JJ, KK] ;
      End ;
    Grid.Zones [2].Columns [JJ + 4].FooterText := Format ('%m', [Total]) ;
    End ;
  End ;
  
Procedure TForm1.GridUnboundEditFetch(Sender: TObject; Zone,
  Column: Integer; RecordNumber: Longint ; Var Value : string) ;
  begin
  If Column In [5..16] Then
    Begin
    Value := CurrToStr (Months [Column - 4, RecordNumber]) ;
    End
  Else
    Begin
    Value := '??????' ;
    End ;

  end;

end.
